# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
#
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# SLH-DSA Signature Algorithm for the CMS
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9814.txt
#

from pyasn1.type import constraint
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280


# Object Identifiers

nistAlgorithms = univ.ObjectIdentifier('2.16.840.1.101.3.4')

sigAlgs = nistAlgorithms + (3, )

id_slh_dsa_sha2_128s = sigAlgs + (20, )

id_slh_dsa_sha2_128f = sigAlgs + (21, )

id_slh_dsa_sha2_192s = sigAlgs + (22, )

id_slh_dsa_sha2_192f = sigAlgs + (23, )

id_slh_dsa_sha2_256s = sigAlgs + (24, )

id_slh_dsa_sha2_256f = sigAlgs + (25, )

id_slh_dsa_shake_128s = sigAlgs + (26, )

id_slh_dsa_shake_128f = sigAlgs + (27, )

id_slh_dsa_shake_192s = sigAlgs + (28, )

id_slh_dsa_shake_192f = sigAlgs + (29, )

id_slh_dsa_shake_256s = sigAlgs + (30, )

id_slh_dsa_shake_256f = sigAlgs + (31, )


# Signature Algorithm Identifier

sa_slh_dsa_sha2_128s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_128s['algorithm'] = id_slh_dsa_sha2_128s
# sa_slh_dsa_sha2_128s['parameters'] is always absent

sa_slh_dsa_sha2_128f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_128f['algorithm'] = id_slh_dsa_sha2_128f
# sa_slh_dsa_sha2_128f['parameters'] is always absent

sa_slh_dsa_sha2_192s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_192s['algorithm'] = id_slh_dsa_sha2_192s
# sa_slh_dsa_sha2_192s['parameters'] is always absent

sa_slh_dsa_sha2_192f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_192f['algorithm'] = id_slh_dsa_sha2_192f
# sa_slh_dsa_sha2_192f['parameters'] is always absent

sa_slh_dsa_sha2_256s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_256s['algorithm'] = id_slh_dsa_sha2_256s
# sa_slh_dsa_sha2_256s['parameters'] is always absent

sa_slh_dsa_sha2_256f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_sha2_256f['algorithm'] = id_slh_dsa_sha2_256f
# sa_slh_dsa_sha2_256f['parameters'] is always absent

sa_slh_dsa_shake_128s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_128s['algorithm'] = id_slh_dsa_shake_128s
# sa_slh_dsa_shake_128s['parameters'] is always absent

sa_slh_dsa_shake_128f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_128f['algorithm'] = id_slh_dsa_shake_128f
# sa_slh_dsa_shake_128f['parameters'] is always absent

sa_slh_dsa_shake_192s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_192s['algorithm'] = id_slh_dsa_shake_192s
# sa_slh_dsa_shake_192s['parameters'] is always absent

sa_slh_dsa_shake_192f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_192f['algorithm'] = id_slh_dsa_shake_192f
# sa_slh_dsa_shake_192f['parameters'] is always absent

sa_slh_dsa_shake_256s = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_256s['algorithm'] = id_slh_dsa_shake_256s
# sa_slh_dsa_shake_256s['parameters'] is always absent

sa_slh_dsa_shake_256f = rfc5280.AlgorithmIdentifier()
sa_slh_dsa_shake_256f['algorithm'] = id_slh_dsa_shake_256f
# sa_slh_dsa_shake_256f['parameters'] is always absent


# Public Keys

pk_slh_dsa_sha2_128s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_128s['algorithm'] = sa_slh_dsa_sha2_128s
# pk_slh_dsa_sha2_128s['subjectPublicKey'] is the raw public key

pk_slh_dsa_sha2_128f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_128f['algorithm'] = sa_slh_dsa_sha2_128f
# pk_slh_dsa_sha2_128f['subjectPublicKey'] is the raw public key

pk_slh_dsa_sha2_192s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_192s['algorithm'] = sa_slh_dsa_sha2_192s
# pk_slh_dsa_sha2_192s['subjectPublicKey'] is the raw public key

pk_slh_dsa_sha2_192f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_192f['algorithm'] = sa_slh_dsa_sha2_192f
# pk_slh_dsa_sha2_192f['subjectPublicKey'] is the raw public key

pk_slh_dsa_sha2_256s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_256s['algorithm'] = sa_slh_dsa_sha2_256s
# pk_slh_dsa_sha2_256s['subjectPublicKey'] is the raw public key

pk_slh_dsa_sha2_256f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_sha2_256f['algorithm'] = sa_slh_dsa_sha2_256f
# pk_slh_dsa_sha2_256f['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_128s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_128s['algorithm'] = sa_slh_dsa_shake_128s
# pk_slh_dsa_shake_128s['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_128f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_128f['algorithm'] = sa_slh_dsa_shake_128f
# pk_slh_dsa_shake_128f['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_192s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_192s['algorithm'] = sa_slh_dsa_shake_192s
# pk_slh_dsa_shake_192s['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_192f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_192f['algorithm'] = sa_slh_dsa_shake_192f
# pk_slh_dsa_shake_192f['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_256s = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_256s['algorithm'] = sa_slh_dsa_shake_256s
# pk_slh_dsa_shake_256s['subjectPublicKey'] is the raw public key

pk_slh_dsa_shake_256f = rfc5280.SubjectPublicKeyInfo()
pk_slh_dsa_shake_256f['algorithm'] = sa_slh_dsa_shake_256f
# pk_slh_dsa_shake_256f['subjectPublicKey'] is the raw public key


# Private key outside an asymmetric key package

class SLH_DSA_PrivateKey(univ.OctetString):
    subtypeSpec = constraint.ConstraintsUnion(
        constraint.ValueSizeConstraint(64, 64),
        constraint.ValueSizeConstraint(96, 96),
        constraint.ValueSizeConstraint(128, 128) )


# Public key outside of a certificate

class SLH_DSA_PublicKey(univ.OctetString):
    subtypeSpec = constraint.ConstraintsUnion(
        constraint.ValueSizeConstraint(32, 32),
        constraint.ValueSizeConstraint(48, 48),
        constraint.ValueSizeConstraint(64, 64) )


# No need to add the id_alg_slh_dsa_* object identifiers to the
# algorithmIdentifierMap or the smimeCapabilityMap because the
# parameters are always absent.



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
# Modified by Russ Housley to use opentypemap.
#
# Copyright (c) 2020-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# The ARIA Encryption Algorithm
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5794.txt
# https://www.rfc-editor.org/errata/eid2064
#

from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap


algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')


# Import from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Object Identifiers

OID = univ.ObjectIdentifier


id_algorithm = univ.ObjectIdentifier((1, 2, 410, 200046, 1,))

id_sea = id_algorithm + (1,)

id_pad = id_algorithm + (2,)


id_aria128_ecb = id_sea + (1,)

id_aria128_cbc = id_sea + (2,)

id_aria128_cfb = id_sea + (3,)

id_aria128_ofb = id_sea + (4,)

id_aria128_ctr = id_sea + (5,)

id_aria192_ecb = id_sea + (6,)

id_aria192_cbc = id_sea + (7,)

id_aria192_cfb = id_sea + (8,)

id_aria192_ofb = id_sea + (9,)

id_aria192_ctr = id_sea + (10,)

id_aria256_ecb = id_sea + (11,)

id_aria256_cbc = id_sea + (12,)

id_aria256_cfb = id_sea + (13,)

id_aria256_ofb = id_sea + (14,)

id_aria256_ctr = id_sea + (15,)

id_aria128_cmac = id_sea + (21,)

id_aria192_cmac = id_sea + (22,)

id_aria256_cmac = id_sea + (23,)

id_aria128_ocb2 = id_sea + (31,)

id_aria192_ocb2 = id_sea + (32,)

id_aria256_ocb2 = id_sea + (33,)

id_aria128_gcm = id_sea + (34,)

id_aria192_gcm = id_sea + (35,)

id_aria256_gcm = id_sea + (36,)

id_aria128_ccm = id_sea + (37,)

id_aria192_ccm = id_sea + (38,)

id_aria256_ccm = id_sea + (39,)

id_aria128_kw = id_sea + (40,)

id_aria192_kw = id_sea + (41,)

id_aria256_kw = id_sea + (42,)

id_aria128_kwp = id_sea + (43,)

id_aria192_kwp = id_sea + (44,)

id_aria256_kwp = id_sea + (45,)


# Relative OIDs

id_pad_null = univ.RelativeOID('0') # no padding algorithms identified

id_pad_1 = univ.RelativeOID('1')    # padding method 2 of ISO/IEC 9797-1:1999


# Parameters

class AriaPadAlgo(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('specifiedPadAlgo', univ.RelativeOID()),
        namedtype.NamedType('generalPadAlgo', univ.ObjectIdentifier())
    )


default_aria_pad_algo_null = AriaPadAlgo()
default_aria_pad_algo_null['specifiedPadAlgo'] = id_pad_null


default_aria_pad_algo_1 = AriaPadAlgo()
default_aria_pad_algo_1['specifiedPadAlgo'] = id_pad_1


class AriaEcbParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('padAlgo', default_aria_pad_algo_null)
    )


class AriaCbcParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('m', univ.Integer().subtype(value=1)),
        namedtype.DefaultedNamedType('padAlgo', default_aria_pad_algo_1)
    )


class AriaCfbParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('r', univ.Integer()),  #  128<=r<=128*1024
        namedtype.NamedType('k', univ.Integer()),  #  1<=k<=128
        namedtype.NamedType('j', univ.Integer()),  #  1<=j<=k
        namedtype.DefaultedNamedType('padAlgo', default_aria_pad_algo_null)
    )


class AriaOfbParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('j', univ.Integer()),  #  1<=j<=128
        namedtype.DefaultedNamedType('padAlgo', default_aria_pad_algo_null)
    )


class AriaCtrParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('j', univ.Integer()),  #  1<=j<=128
        namedtype.DefaultedNamedType('padAlgo', default_aria_pad_algo_null)
    )


class AriaCmacParameters(univ.Integer):
    pass


class AriaOcb2Parameters(univ.Integer):
    pass


class AriaGcmParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('s', univ.Integer()),
        namedtype.NamedType('t', univ.Integer())
    )


class AriaCcmParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('w', univ.Integer().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                2, 3, 4, 5, 6, 7, 8))),
        namedtype.NamedType('t', univ.Integer().subtype(
            subtypeSpec=constraint.SingleValueConstraint(
                32, 48, 64, 80, 96, 112, 128)))
    )


# Algorithm Identifiers

class AriaModeOfOperation(AlgorithmIdentifier):
    pass


aria128ecb = AlgorithmIdentifier()
aria128ecb['algorithm'] = id_aria128_ecb
aria128ecb['parameters'] = AriaEcbParameters() 


aria128cbc = AlgorithmIdentifier()
aria128cbc['algorithm'] = id_aria128_cbc
aria128cbc['parameters'] = AriaCbcParameters() 


aria128cfb = AlgorithmIdentifier()
aria128cfb['algorithm'] = id_aria128_cfb
aria128cfb['parameters'] = AriaCfbParameters() 


aria128ofb = AlgorithmIdentifier()
aria128ofb['algorithm'] = id_aria128_ofb
aria128ofb['parameters'] = AriaOfbParameters() 


aria128ctr = AlgorithmIdentifier()
aria128ctr['algorithm'] = id_aria128_ctr
aria128ctr['parameters'] = AriaCtrParameters() 


aria192ecb = AlgorithmIdentifier()
aria192ecb['algorithm'] = id_aria192_ecb
aria192ecb['parameters'] = AriaEcbParameters() 


aria192cbc = AlgorithmIdentifier()
aria192cbc['algorithm'] = id_aria192_cbc
aria192cbc['parameters'] = AriaCbcParameters() 


aria192cfb = AlgorithmIdentifier()
aria192cfb['algorithm'] = id_aria192_cfb
aria192cfb['parameters'] = AriaCfbParameters() 


aria192ofb = AlgorithmIdentifier()
aria192ofb['algorithm'] = id_aria192_ofb
aria192ofb['parameters'] = AriaOfbParameters() 


aria192ctr = AlgorithmIdentifier()
aria192ctr['algorithm'] = id_aria192_ctr
aria192ctr['parameters'] = AriaCtrParameters() 


aria256ecb = AlgorithmIdentifier()
aria256ecb['algorithm'] = id_aria256_ecb
aria256ecb['parameters'] = AriaEcbParameters() 


aria256cbc = AlgorithmIdentifier()
aria256cbc['algorithm'] = id_aria256_cbc
aria256cbc['parameters'] = AriaCbcParameters() 


aria256cfb = AlgorithmIdentifier()
aria256cfb['algorithm'] = id_aria256_cfb
aria256cfb['parameters'] = AriaCfbParameters() 


aria256ofb = AlgorithmIdentifier()
aria256ofb['algorithm'] = id_aria256_ofb
aria256ofb['parameters'] = AriaOfbParameters() 


aria256ctr = AlgorithmIdentifier()
aria256ctr['algorithm'] = id_aria256_ctr
aria256ctr['parameters'] = AriaCtrParameters() 


aria128cmac = AlgorithmIdentifier()
aria128cmac['algorithm'] = id_aria128_cmac
aria128cmac['parameters'] = AriaCmacParameters() 


aria192cmac = AlgorithmIdentifier()
aria192cmac['algorithm'] = id_aria192_cmac
aria192cmac['parameters'] = AriaCmacParameters() 


aria256cmac = AlgorithmIdentifier()
aria256cmac['algorithm'] = id_aria256_cmac
aria256cmac['parameters'] = AriaCmacParameters() 


aria128ocb2 = AlgorithmIdentifier()
aria128ocb2['algorithm'] = id_aria128_ocb2
aria128ocb2['parameters'] = AriaOcb2Parameters() 


aria192ocb2 = AlgorithmIdentifier()
aria192ocb2['algorithm'] = id_aria192_ocb2
aria192ocb2['parameters'] = AriaOcb2Parameters() 


aria256ocb2 = AlgorithmIdentifier()
aria256ocb2['algorithm'] = id_aria256_ocb2
aria256ocb2['parameters'] = AriaOcb2Parameters() 


aria128gcm = AlgorithmIdentifier()
aria128gcm['algorithm'] = id_aria128_gcm
aria128gcm['parameters'] = AriaGcmParameters() 


aria192gcm = AlgorithmIdentifier()
aria192gcm['algorithm'] = id_aria192_gcm
aria192gcm['parameters'] = AriaGcmParameters() 


aria256gcm = AlgorithmIdentifier()
aria256gcm['algorithm'] = id_aria256_gcm
aria256gcm['parameters'] = AriaGcmParameters() 


aria128ccm = AlgorithmIdentifier()
aria128ccm['algorithm'] = id_aria128_ccm
aria128ccm['parameters'] = AriaCcmParameters() 


aria192ccm = AlgorithmIdentifier()
aria192ccm['algorithm'] = id_aria192_ccm
aria192ccm['parameters'] = AriaCcmParameters() 


aria256ccm = AlgorithmIdentifier()
aria256ccm['algorithm'] = id_aria256_ccm
aria256ccm['parameters'] = AriaCcmParameters() 


aria128kw = AlgorithmIdentifier()
aria128kw['algorithm'] = id_aria128_kw


aria192kw = AlgorithmIdentifier()
aria192kw['algorithm'] = id_aria192_kw


aria256kw = AlgorithmIdentifier()
aria256kw['algorithm'] = id_aria256_kw


aria128kwp = AlgorithmIdentifier()
aria128kwp['algorithm'] = id_aria128_kwp


aria192kwp = AlgorithmIdentifier()
aria192kwp['algorithm'] = id_aria192_kwp


aria256kwp = AlgorithmIdentifier()
aria256kwp['algorithm'] = id_aria256_kwp


# Update the Algorithm Identifier map

_algorithmIdentifierMapUpdate = {
    id_aria128_ecb: AriaEcbParameters(),
    id_aria128_cbc: AriaCbcParameters(),
    id_aria128_cfb: AriaCfbParameters(),
    id_aria128_ofb: AriaOfbParameters(),
    id_aria128_ctr: AriaCtrParameters(),
    id_aria192_ecb: AriaEcbParameters(),
    id_aria192_cbc: AriaCbcParameters(),
    id_aria192_cfb: AriaCfbParameters(),
    id_aria192_ofb: AriaOfbParameters(),
    id_aria192_ctr: AriaCtrParameters(),
    id_aria256_ecb: AriaEcbParameters(),
    id_aria256_cbc: AriaCbcParameters(),
    id_aria256_cfb: AriaCfbParameters(),
    id_aria256_ofb: AriaOfbParameters(),
    id_aria256_ctr: AriaCtrParameters(),
    id_aria128_cmac: AriaCmacParameters(),
    id_aria192_cmac: AriaCmacParameters(),
    id_aria256_cmac: AriaCmacParameters(),
    id_aria128_ocb2: AriaOcb2Parameters(),
    id_aria192_ocb2: AriaOcb2Parameters(),
    id_aria256_ocb2: AriaOcb2Parameters(),
    id_aria128_gcm: AriaGcmParameters(),
    id_aria192_gcm: AriaGcmParameters(),
    id_aria256_gcm: AriaGcmParameters(),
    id_aria128_ccm: AriaCcmParameters(),
    id_aria192_ccm: AriaCcmParameters(),
    id_aria256_ccm: AriaCcmParameters(),
}

algorithmIdentifierMap.update(_algorithmIdentifierMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with some help from asn1ate v.0.6.0
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Server-Based Certificate Validation Protocol (SCVP)
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc5055.txt
# https://www.rfc-editor.org/rfc/rfc5912.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import opentype
from pyasn1.type import tag
from pyasn1.type import useful
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc6960
from pyasn1_alt_modules import rfc3281
from pyasn1_alt_modules import opentypemap

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')

otherRevInfoFormatMap = opentypemap.get('otherRevInfoFormatMap')

scvpValidationPolMap = opentypemap.get('scvpValidationPolMap')

scvpValidationAlgMap = opentypemap.get('scvpValidationAlgMap')

scvpWantBackMap = opentypemap.get('scvpWantBackMap')

MAX = float('inf')


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Attribute = rfc5280.Attribute

Certificate = rfc5280.Certificate

Extensions = rfc5280.Extensions

CertificateList = rfc5280.CertificateList

CertificateSerialNumber = rfc5280.CertificateSerialNumber

GeneralNames = rfc5280.GeneralNames

GeneralName = rfc5280.GeneralName

KeyUsage = rfc5280.KeyUsage

KeyPurposeId = rfc5280.KeyPurposeId

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo


# Imports from RFC 3281

AttributeCertificate = rfc3281.AttributeCertificate


# Imports from RFC 6960

OCSPResponse = rfc6960.OCSPResponse


# Imports from RFC 5652

ContentInfo = rfc5652.ContentInfo


# Server-Based Certificate Validation Protocol

sha_1 = univ.ObjectIdentifier((1, 3, 14, 3, 2, 26))

algid_SHA1 = AlgorithmIdentifier()
algid_SHA1['algorithm'] = sha_1
algid_SHA1['parameters'] = "\x05\x00"


class SCVPIssuerSerial(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('issuer', GeneralNames()),
        namedtype.NamedType('serialNumber', CertificateSerialNumber())
    )


class SCVPCertID(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('certHash', univ.OctetString()),
        namedtype.NamedType('issuerSerial', SCVPIssuerSerial()),
        namedtype.DefaultedNamedType('hashAlgorithm', algid_SHA1)
    )


class ACReference(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('attrCert', AttributeCertificate().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 2))),
        namedtype.NamedType('acRef', SCVPCertID().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 3)))
    )


class AuthPolicy(univ.ObjectIdentifier):
    pass


class ValidationPolRef(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('valPolId', univ.ObjectIdentifier()),
        namedtype.OptionalNamedType('valPolParams', univ.Any(),
            openType=opentype.OpenType('valPolId', scvpValidationPolMap))
    )


class PKCReference(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('cert', Certificate().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 0))),
        namedtype.NamedType('pkcRef', SCVPCertID().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 1)))
    )


class TrustAnchors(univ.SequenceOf):
    componentType = PKCReference()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class ValidationAlg(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('valAlgId', univ.ObjectIdentifier()),
        namedtype.OptionalNamedType('parameters', univ.Any(),
            openType=opentype.OpenType('valPolId', scvpValidationAlgMap))
    )


class ValidationPolicy(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('validationPolRef', ValidationPolRef()),
        namedtype.OptionalNamedType('validationAlg',
            ValidationAlg().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('userPolicySet',
            univ.SequenceOf(componentType=univ.ObjectIdentifier()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('inhibitPolicyMapping',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('requireExplicitPolicy',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 3))),
        namedtype.OptionalNamedType('inhibitAnyPolicy',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 4))),
        namedtype.OptionalNamedType('trustAnchors',
            TrustAnchors().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 5))),
        namedtype.OptionalNamedType('keyUsages',
            univ.SequenceOf(componentType=KeyUsage()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext,
                    tag.tagFormatSimple, 6))),
        namedtype.OptionalNamedType('extendedKeyUsages',
            univ.SequenceOf(componentType=KeyPurposeId()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext,
                    tag.tagFormatSimple, 7))),
        namedtype.OptionalNamedType('specifiedKeyUsages',
            univ.SequenceOf(componentType=KeyPurposeId()).subtype(
                implicitTag=tag.Tag(tag.tagClassContext,
                    tag.tagFormatSimple, 8)))
    )


class CertChecks(univ.SequenceOf):
    componentType = univ.ObjectIdentifier()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class ResponseFlags(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('fullRequestInResponse',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 0)).subtype(value=0)),
        namedtype.DefaultedNamedType('responseValidationPolByRef',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 1)).subtype(value=1)),
        namedtype.DefaultedNamedType('protectResponse',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 2)).subtype(value=1)),
        namedtype.DefaultedNamedType('cachedResponse',
            univ.Boolean().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 3)).subtype(value=1))
    )


class CertBundle(univ.SequenceOf):
    componentType = Certificate()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class OtherRevInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('riType', univ.ObjectIdentifier()),
        namedtype.NamedType('riValue', univ.Any(),
            openType=opentype.OpenType('riType', otherRevInfoFormatMap))
    )


class RevocationInfo(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('crl', CertificateList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 0))),
        namedtype.NamedType('delta-crl', CertificateList().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 1))),
        namedtype.NamedType('ocsp', OCSPResponse().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 2))),
        namedtype.NamedType('other', OtherRevInfo().subtype(
            implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 3)))
    )


class RevocationInfos(univ.SequenceOf):
    componentType = RevocationInfo()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class CertReferences(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('pkcRefs', univ.SequenceOf(
            componentType=PKCReference()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 0))),
        namedtype.NamedType('acRefs', univ.SequenceOf(
            componentType=ACReference()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 1)))
    )


class WantBack(univ.SequenceOf):
    componentType = univ.ObjectIdentifier()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class Query(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('queriedCerts', CertReferences()),
        namedtype.NamedType('checks', CertChecks()),
        namedtype.OptionalNamedType('wantBack',
            WantBack().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatSimple, 1))),
        namedtype.NamedType('validationPolicy', ValidationPolicy()),
        namedtype.OptionalNamedType('responseFlags', ResponseFlags()),
        namedtype.OptionalNamedType('serverContextInfo',
            univ.OctetString().subtype(implicitTag=tag.Tag(
                 tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('validationTime',
            useful.GeneralizedTime().subtype(implicitTag=tag.Tag(
                 tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.OptionalNamedType('intermediateCerts',
            CertBundle().subtype(implicitTag=tag.Tag(
                 tag.tagClassContext, tag.tagFormatSimple, 4))),
        namedtype.OptionalNamedType('revInfos',
            RevocationInfos().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 5))),
        namedtype.OptionalNamedType('producedAt',
            useful.GeneralizedTime().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 6))),
        namedtype.OptionalNamedType('queryExtensions',
            Extensions().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 7)))
    )



class CVRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('cvRequestVersion',
            univ.Integer().subtype(value=1)),
        namedtype.NamedType('query', Query()),
        namedtype.OptionalNamedType('requestorRef',
            GeneralNames().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('requestNonce',
            univ.OctetString().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('requestorName',
            GeneralName().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('responderName',
            GeneralName().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.OptionalNamedType('requestExtensions',
            Extensions().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 4))),
        namedtype.OptionalNamedType('signatureAlg',
            AlgorithmIdentifier().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 5))),
        namedtype.OptionalNamedType('hashAlg',
            univ.ObjectIdentifier().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 6))),
        namedtype.OptionalNamedType('requestorText',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 256)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 7)))
    )


class RespValidationPolicy(ValidationPolicy):
    pass


class CertReference(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('pkc', PKCReference()),
        namedtype.NamedType('ac', ACReference())
    )


class ReplyStatus(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ('success', 0),
        ('malformedPKC', 1),
        ('malformedAC', 2),
        ('unavailableValidationTime', 3),
        ('referenceCertHashFail', 4),
        ('certPathConstructFail', 5),
        ('certPathNotValid', 6),
        ('certPathNotValidNow', 7),
        ('wantBackUnsatisfied', 8)
    )


class ReplyCheck(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('check', univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType('status', univ.Integer().subtype(value=0))
    )


class ReplyChecks(univ.SequenceOf):
    componentType = ReplyCheck()


class ReplyWantBack(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('wb', univ.ObjectIdentifier()),
        namedtype.NamedType('value', univ.OctetString())
    )


class ReplyWantBacks(univ.SequenceOf):
    componentType = ReplyWantBack()


class CertReply(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('cert', CertReference()),
        namedtype.DefaultedNamedType('replyStatus',
            ReplyStatus().subtype(value='success')),
        namedtype.NamedType('replyValTime', useful.GeneralizedTime()),
        namedtype.NamedType('replyChecks', ReplyChecks()),
        namedtype.NamedType('replyWantBacks', ReplyWantBacks()),
        namedtype.OptionalNamedType('validationErrors',
            univ.SequenceOf(componentType=univ.ObjectIdentifier()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('nextUpdate',
            useful.GeneralizedTime().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('certReplyExtensions',
            Extensions().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 2)))
    )


class ReplyObjects(univ.SequenceOf):
    componentType = CertReply()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class HashValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('algorithm', algid_SHA1),
        namedtype.NamedType('value', univ.OctetString())
    )


class RequestReference(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('requestHash',
            HashValue().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 0))),
        namedtype.NamedType('fullRequest',
            CVRequest().subtype(implicitTag=tag.Tag(tag.tagClassContext,
                tag.tagFormatConstructed, 1)))
    )


class CVStatusCode(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ('okay', 0),
        ('skipUnrecognizedItems', 1),
        ('tooBusy', 10),
        ('invalidRequest', 11),
        ('internalError', 12),
        ('badStructure', 20),
        ('unsupportedVersion', 21),
        ('abortUnrecognizedItems', 22),
        ('unrecognizedSigKey', 23),
        ('badSignatureOrMAC', 24),
        ('unableToDecode', 25),
        ('notAuthorized', 26),
        ('unsupportedChecks', 27),
        ('unsupportedWantBacks', 28),
        ('unsupportedSignatureOrMAC', 29),
        ('invalidSignatureOrMAC', 30),
        ('protectedResponseUnsupported', 31),
        ('unrecognizedResponderName', 32),
        ('relayingLoop', 40),
        ('unrecognizedValPol', 50),
        ('unrecognizedValAlg', 51),
        ('fullRequestInResponseUnsupported', 52),
        ('fullPolResponseUnsupported', 53),
        ('inhibitPolicyMappingUnsupported', 54),
        ('requireExplicitPolicyUnsupported', 55),
        ('inhibitAnyPolicyUnsupported', 56),
        ('validationTimeUnsupported', 57),
        ('unrecognizedCritQueryExt', 63),
        ('unrecognizedCritRequestExt', 64)
    )


class ResponseStatus(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('statusCode',
            CVStatusCode().subtype(value='okay')),
        namedtype.OptionalNamedType('errorMessage',
            char.UTF8String())
    )


class CVResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('cvResponseVersion', univ.Integer()),
        namedtype.NamedType('serverConfigurationID', univ.Integer()),
        namedtype.NamedType('producedAt', useful.GeneralizedTime()),
        namedtype.NamedType('responseStatus', ResponseStatus()),
        namedtype.OptionalNamedType('respValidationPolicy',
            RespValidationPolicy().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('requestRef',
            RequestReference().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 1))),
        namedtype.OptionalNamedType('requestorRef',
            GeneralNames().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('requestorName',
            GeneralNames().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.OptionalNamedType('replyObjects',
            ReplyObjects().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 4))),
        namedtype.OptionalNamedType('respNonce',
            univ.OctetString().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 5))),
        namedtype.OptionalNamedType('serverContextInfo',
            univ.OctetString().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 6))),
        namedtype.OptionalNamedType('cvResponseExtensions',
            Extensions().subtype(implicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatSimple, 7))),
        namedtype.OptionalNamedType('requestorText',
            char.UTF8String().subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, 256)).subtype(
                    implicitTag=tag.Tag(tag.tagClassContext,
                        tag.tagFormatSimple, 8)))
    )


class CertBundles(univ.SequenceOf):
    componentType = CertBundle()
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


class KeyAgreePublicKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('algorithm', AlgorithmIdentifier()),
        namedtype.NamedType('publicKey', univ.BitString()),
        namedtype.NamedType('macAlgorithm', AlgorithmIdentifier()),
        namedtype.OptionalNamedType('kDF', AlgorithmIdentifier())
    )


class NameValidationAlgParms(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('nameCompAlgId', univ.ObjectIdentifier()),
        namedtype.NamedType('validationNames', GeneralNames())
    )

class ResponseTypes(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ('cached-only', 0),
        ('non-cached-only', 1),
        ('cached-and-non-cached', 2)
    )


class RevInfoWantBack(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('revocationInfo', RevocationInfos()),
        namedtype.OptionalNamedType('extraCerts', CertBundle())
    )


class RevocationInfoTypes(univ.BitString):
    namedValues = namedval.NamedValues(
        ('fullCRLs', 0),
        ('deltaCRLs', 1),
        ('indirectCRLs', 2),
        ('oCSPResponses', 3)
    )


class SCVPResponses(univ.SequenceOf):
    componentType = ContentInfo()


class ValPolRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('vpRequestVersion',
            univ.Integer().subtype(value=1)),
        namedtype.NamedType('requestNonce', univ.OctetString())
    )


class ValPolResponse(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('vpResponseVersion', univ.Integer()),
        namedtype.NamedType('maxCVRequestVersion', univ.Integer()),
        namedtype.NamedType('maxVPRequestVersion', univ.Integer()),
        namedtype.NamedType('serverConfigurationID', univ.Integer()),
        namedtype.NamedType('thisUpdate', useful.GeneralizedTime()),
        namedtype.OptionalNamedType('nextUpdate', useful.GeneralizedTime()),
        namedtype.NamedType('supportedChecks', CertChecks()),
        namedtype.NamedType('supportedWantBacks', WantBack()),
        namedtype.NamedType('validationPolicies',
            univ.SequenceOf(componentType=univ.ObjectIdentifier())),
        namedtype.NamedType('validationAlgs',
            univ.SequenceOf(componentType=univ.ObjectIdentifier())),
        namedtype.NamedType('authPolicies',
            univ.SequenceOf(componentType=AuthPolicy())),
        namedtype.NamedType('responseTypes', ResponseTypes()),
        namedtype.NamedType('defaultPolicyValues', RespValidationPolicy()),
        namedtype.NamedType('revocationInfoTypes', RevocationInfoTypes()),
        namedtype.NamedType('signatureGeneration',
            univ.SequenceOf(componentType=AlgorithmIdentifier())),
        namedtype.NamedType('signatureVerification',
            univ.SequenceOf(componentType=AlgorithmIdentifier())),
        namedtype.NamedType('hashAlgorithms',
            univ.SequenceOf(componentType=univ.ObjectIdentifier()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.OptionalNamedType('serverPublicKeys',
            univ.SequenceOf(componentType=KeyAgreePublicKey())),
        namedtype.DefaultedNamedType('clockSkew',
            univ.Integer().subtype(value=10)),
        namedtype.OptionalNamedType('requestNonce', univ.OctetString())
    )


# SCVP Check Identifiers

id_stc = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 17))

id_stc_build_pkc_path = id_stc + (1,)

id_stc_build_valid_pkc_path = id_stc + (2,)

id_stc_build_status_checked_pkc_path = id_stc + (3,)

id_stc_build_aa_path = id_stc + (4,)

id_stc_build_valid_aa_path = id_stc + (5,)

id_stc_build_status_checked_aa_path = id_stc + (6,)

id_stc_status_check_ac_and_build_status_checked_aa_path = id_stc + (7,)


# SCVP WantBack Identifiers

id_swb = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 18))

id_swb_pkc_best_cert_path = id_swb + (1,)

id_swb_pkc_revocation_info = id_swb + (2,)

id_swb_pkc_public_key_info = id_swb + (4,)

id_swb_aa_cert_path = id_swb + (5,)

id_swb_aa_revocation_info = id_swb + (6,)

id_swb_ac_revocation_info = id_swb + (7,)

id_swb_relayed_responses = id_swb + (9,)

id_swb_pkc_cert = id_swb + (10,)

id_swb_ac_cert = id_swb + (11,)

id_swb_pkc_all_cert_paths = id_swb + (12,)

id_swb_pkc_ee_revocation_info = id_swb + (13,)

id_swb_pkc_CAs_revocation_info = id_swb + (14,)


# SCVP Validation Policy and Algorithm Identifiers

id_svp = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 19))

id_svp_defaultValPolicy = id_svp + (1,)

id_svp_nameValAlg = id_svp + (2,)

id_svp_basicValAlg = id_svp + (3,)

id_nva_dnCompAlg = id_svp + (4,)


# SCVP Basic Validation Algorithm Errors

id_bvae = univ.ObjectIdentifier(id_svp_basicValAlg)

id_bvae_expired = id_bvae + (1,)

id_bvae_not_yet_valid = id_bvae + (2,)

id_bvae_wrongTrustAnchor = id_bvae + (3,)

id_bvae_noValidCertPath = id_bvae + (4,)

id_bvae_revoked = id_bvae + (5,)

id_bvae_invalidKeyPurpose = id_bvae + (9,)

id_bvae_invalidKeyUsage = id_bvae + (10,)

id_bvae_invalidCertPolicy = id_bvae + (11,)


# SCVP Name Validation Algorithm Errors

id_nvae = univ.ObjectIdentifier(id_svp_nameValAlg)

id_nvae_name_mismatch = id_nvae + (1,)

id_nvae_no_name = id_nvae + (2,)

id_nvae_unknown_alg = id_nvae + (3,)

id_nvae_bad_name = id_nvae + (4,)

id_nvae_bad_name_type = id_nvae + (5,)

id_nvae_mixed_names = id_nvae + (6,)


# SCVP Extended Key Usage Key Purpose Identifiers

id_kp = univ.ObjectIdentifier((1, 3, 6, 1, 5, 5, 7, 3))

id_kp_scvpServer = id_kp + (15,)

id_kp_scvpClient = id_kp + (16,)


# CMS Content Types

id_ct = univ.ObjectIdentifier((1, 2, 840, 113549, 1, 9, 16, 1))

id_ct_scvp_certValRequest = id_ct + (10,)

id_ct_scvp_certValResponse = id_ct + (11,)

id_ct_scvp_valPolRequest = id_ct + (12,)

id_ct_scvp_valPolResponse = id_ct + (13,)


# Update the Content Types Map

_cmsContentTypesMapUpdate = {
    id_ct_scvp_certValRequest: CVRequest(),
    id_ct_scvp_certValResponse: CVResponse(),
    id_ct_scvp_valPolRequest: ValPolRequest(),
    id_ct_scvp_valPolResponse: ValPolResponse(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)


# id_svp_defaultValPolicy: parameters MUST be absent
# so there is nothing to add to scvpValidationPolMap


# Update the SCVP Validation Algorithm Map

_scvpValidationAlgMapUpdate = {
    # id_svp_basicValAlg: parameters MUST be absent
    id_svp_nameValAlg: NameValidationAlgParms(),
}

scvpValidationAlgMap.update(_scvpValidationAlgMapUpdate)


# Update the SCVP Want Back Map

_scvpWantBackMapUpdate = {
    id_swb_pkc_best_cert_path: CertBundle(),
    id_swb_pkc_revocation_info: RevInfoWantBack(),
    id_swb_pkc_public_key_info: SubjectPublicKeyInfo(),
    id_swb_aa_cert_path: CertBundle(),
    id_swb_aa_revocation_info: RevInfoWantBack(),
    id_swb_ac_revocation_info: RevInfoWantBack(),
    id_swb_relayed_responses: SCVPResponses(),
    id_swb_pkc_cert: Certificate(),
    id_swb_ac_cert: AttributeCertificate(),
    id_swb_pkc_all_cert_paths: CertBundles(),
    id_swb_pkc_ee_revocation_info: RevInfoWantBack(),
    id_swb_pkc_CAs_revocation_info: RevInfoWantBack(),
}

scvpWantBackMap.update(_scvpWantBackMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with some assistance from asn1ate v.0.6.0.
# Modified by Russ Housley to include the opentypemap manager.
#
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Autonomic Control Plane (ACP) Node Name in X.509 Certificates
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc8994.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

otherNamesMap = opentypemap.get('otherNamesMap')

MAX = float('inf')


# Autonomic Control Plane (ACP) Node Name

id_pkix = rfc5280.id_pkix

id_on = id_pkix + (8, )

id_on_AcpNodeName = id_on + (10, )


class AcpNodeName(char.IA5String):
    subtypeSpec = constraint.ValueSizeConstraint(1, MAX)


on_AcpNodeName = rfc5280.AnotherName()
on_AcpNodeName['type-id'] = id_on_AcpNodeName
on_AcpNodeName['value'] = AcpNodeName()


# Update the Other Names Map

_otherNamesMapUpdate = {
    id_on_AcpNodeName: AcpNodeName(),
}

otherNamesMap.update(_otherNamesMapUpdate)



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder
from pyasn1.type import univ

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc3565
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc5751
from pyasn1_alt_modules import rfc9709
from pyasn1_alt_modules import opentypemap


class CMSEnvelopedDataTestCase(unittest.TestCase):
    cms_pem_text = """\
MIG9BgkqhkiG9w0BBwOgga8wgawCAQIxOqI4AgEEMAwEChjH/HKfZsLuGMQwCwYJ
YIZIAWUDBAEFBBjFfwphC6MYnMBpfSw/MIhGYHGEtxwrgukwawYJKoZIhvcNAQcB
MCwGCyqGSIb3DQEJEAMfMB0GCWCGSAFlAwQBAgQQlixF80pCLxf9qVGK+vl5yIAw
YjvrUVNOpWnJBeQDOGEGMWbpy9S5TI0KUfnR6rPZkOD2SfS8lwNkY/ZeMTwdihFB
"""

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cms_pem_text)
        ci, rest = der_decoder(substrate, asn1Spec=rfc5652.ContentInfo())
        self.assertFalse(rest)
        self.assertTrue(ci.prettyPrint())
        self.assertEqual(substrate, der_encoder(ci))

        self.assertEqual(rfc5652.id_envelopedData, ci['contentType'])
        ed, rest = der_decoder(ci['content'], asn1Spec=rfc5652.EnvelopedData())
        self.assertFalse(rest)
        self.assertTrue(ed.prettyPrint())
        self.assertEqual(ci['content'], der_encoder(ed))

        self.assertEqual(2, ed['version'])
        self.assertEqual(4, ed['recipientInfos'][0]['kekri']['version'])
        oid = ed['encryptedContentInfo']['contentEncryptionAlgorithm']['algorithm']
        self.assertEqual(rfc9709.id_alg_cek_hkdf_sha256, oid)

        s = ed['encryptedContentInfo']['contentEncryptionAlgorithm']['parameters']
        p, rest = der_decoder(s, asn1Spec=rfc5280.AlgorithmIdentifier())
        self.assertFalse(rest)
        self.assertTrue(p.prettyPrint())
        self.assertEqual(s, der_encoder(p))

        self.assertEqual(p['algorithm'], rfc3565.id_aes128_CBC)

    def testOpenTypes(self):
        algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')
        self.assertIn(rfc9709.id_alg_cek_hkdf_sha256, algorithmIdentifierMap)
        self.assertIn(rfc3565.id_aes128_CBC, algorithmIdentifierMap)

        substrate = pem.readBase64fromText(self.cms_pem_text)
        ci, rest = der_decoder(substrate,
            asn1Spec=rfc5652.ContentInfo(), decodeOpenTypes=True)
        self.assertFalse(rest)
        self.assertTrue(ci.prettyPrint())
        self.assertEqual(substrate, der_encoder(ci))

        ed = ci['content']
        self.assertEqual(2, ed['version'])
        self.assertEqual(4, ed['recipientInfos'][0]['kekri']['version'])
        oid = ed['encryptedContentInfo']['contentEncryptionAlgorithm']['algorithm']
        self.assertEqual(rfc9709.id_alg_cek_hkdf_sha256, oid)
        p = ed['encryptedContentInfo']['contentEncryptionAlgorithm']['parameters']
        self.assertEqual(p['algorithm'], rfc3565.id_aes128_CBC)


class SMimeCapabilitiesTestCase(unittest.TestCase):
    smimecaps_pem_text = """\
MGswCgYIKoUDBwEBBAEwCgYIKoUDBwEBBAIwIAYHKoUDAgINADAVBAgAAAAAAAAA
AAYJKoUDBwECBQEBMCAGByqFAwICDQEwFQQIAAAAAAAAAAAGCSqFAwcBAgUBATAN
BgsqhkiG9w0BCRADHw==
"""

    def setUp(self):
        self.asn1Spec = rfc5751.SMIMECapabilities()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.smimecaps_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        found = False
        for algid in asn1Object:
            if algid['capabilityID'] == rfc9709.id_alg_cek_hkdf_sha256:
                found = True

        self.assertTrue(found)


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc4357
from pyasn1_alt_modules import rfc9215
from pyasn1_alt_modules import rfc9548

from pyasn1_alt_modules import opentypemap


class GostR34102012256CertificateTestCase(unittest.TestCase):
    pem_text = """\
MIIBJTCB06ADAgECAgEKMAoGCCqFAwcBAQMCMBIxEDAOBgNVBAMTB0V4YW1wbGUw
IBcNMDEwMTAxMDAwMDAwWhgPMjA1MDEyMzEwMDAwMDBaMBIxEDAOBgNVBAMTB0V4
YW1wbGUwXjAXBggqhQMHAQEBATALBgkqhQMHAQIBAQEDQwAEQHQnldS+6ITd8oUP
7APqP68YROAdnaYLZFCTpV4m38OZePWWz01NDGzx0YlD2UST0WuewKFtUS0uEnzE
aRpjGOKjEzARMA8GA1UdEwEB/wQFMAMBAf8wCgYIKoUDBwEBAwIDQQAUC02pEksJ
yw1c6Sjuh0JzoxASlJLsDik2njt5EkhXjB0OHaW+NHxvG1JWx66sIArWSsd6b1s6
DglzGOeubudp
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        spki = asn1Object['tbsCertificate']['subjectPublicKeyInfo']
        pk_encoded = spki['subjectPublicKey'].asOctets()
        ans1Spec = rfc9548.GostR3410_2012_PublicKey()
        pk, rest = der_decoder(pk_encoded, asn1Spec=ans1Spec)
        self.assertFalse(rest)
        self.assertTrue(pk.prettyPrint())
        self.assertEqual(pk_encoded, der_encoder(pk))


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Modified by Russ Housley to import from RFC 5280 instead of RFC 2459.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# PKCS#10 syntax
#
# ASN.1 source from:
# http://tools.ietf.org/html/rfc2314
#
# Sample captures could be obtained with "openssl req" command
#
from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280


# Imports from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier

Attribute = rfc5280.Attribute

Name = rfc5280.Name

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo


# PKCS#10

class Attributes(univ.SetOf):
    componentType = Attribute()


class Version(univ.Integer):
    pass


class CertificationRequestInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version', Version()),
        namedtype.NamedType('subject', Name()),
        namedtype.NamedType('subjectPublicKeyInfo', SubjectPublicKeyInfo()),
        namedtype.NamedType('attributes',
                            Attributes().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)))
    )


class Signature(univ.BitString):
    pass


class SignatureAlgorithmIdentifier(AlgorithmIdentifier):
    pass


class CertificationRequest(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('certificationRequestInfo', CertificationRequestInfo()),
        namedtype.NamedType('signatureAlgorithm', SignatureAlgorithmIdentifier()),
        namedtype.NamedType('signature', Signature())
    )



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2019-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc5652
from pyasn1_alt_modules import rfc6210
from pyasn1_alt_modules import opentypemap


class AuthenticatedDataTestCase(unittest.TestCase):
    pem_text = """\
MIICRQYLKoZIhvcNAQkQAQKgggI0MIICMAIBADGBwDCBvQIBADAmMBIxEDAOBgNVBAMMB0
NhcmxSU0ECEEY0a8eAAFa8EdNuLs1dcdAwDQYJKoZIhvcNAQEBBQAEgYCH70EpEikY7deb
859YJRAWfFondQv1D4NFltw6C1ceheWnlAU0C2WEXr3LUBXZp1/PSte29FnJxu5bXCTn1g
elMm6zNlZNWNd0KadVBcaxi1n8L52tVM5sWFGJPO5cStOyAka2ucuZM6iAnCSkn1Ju7fgU
5j2g3bZ/IM8nHTcygjAKBggrBgEFBQgBAqFPBgsqhkiG9w0BCRADDQRAAQIDBAUGBwgJCg
sMDQ4PEBESEwQVFhcYGRobHB0eHyAhIiMEJSYnKCkqKywtLi8wMTIzBDU2Nzg5Ojs8PT4/
QDArBgkqhkiG9w0BBwGgHgQcVGhpcyBpcyBzb21lIHNhbXBsZSBjb250ZW50LqKBxzAYBg
kqhkiG9w0BCQMxCwYJKoZIhvcNAQcBMBwGCSqGSIb3DQEJBTEPFw0wOTEyMTAyMzI1MDBa
MB8GCSqGSIb3DQEJBDESBBCWaa5hG1eeg+oQK2tJ3cD5MGwGCSqGSIb3DQEJNDFfMF0wTw
YLKoZIhvcNAQkQAw0EQAECAwQFBgcICQoLDA0ODxAREhMEFRYXGBkaGxwdHh8gISIjBCUm
JygpKissLS4vMDEyMwQ1Njc4OTo7PD0+P0CiCgYIKwYBBQUIAQIEFLjUxQ9PJFzFnWraxb
EIbVbg2xql
"""

    def setUp(self):
        self.asn1Spec = rfc5652.ContentInfo()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)
        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        self.assertEqual(rfc5652.id_ct_authData, asn1Object['contentType'])

        ad, rest = der_decoder(asn1Object['content'],
            asn1Spec=rfc5652.AuthenticatedData())
        self.assertFalse(rest)
        self.assertTrue(ad.prettyPrint())
        self.assertEqual(asn1Object['content'], der_encoder(ad))
        
        self.assertEqual(0, ad['version'])
        self.assertEqual(
            rfc6210.id_alg_MD5_XOR_EXPERIMENT, ad['digestAlgorithm']['algorithm'])
            
        algorithmIdentifierMap = opentypemap.get('algorithmIdentifierMap')
        mac_alg_p, rest = der_decoder(ad['digestAlgorithm']['parameters'],
            asn1Spec=algorithmIdentifierMap[ad['digestAlgorithm']['algorithm']])
        self.assertFalse(rest)
        self.assertTrue(mac_alg_p.prettyPrint())
        self.assertEqual(
            ad['digestAlgorithm']['parameters'], der_encoder(mac_alg_p))

        self.assertEqual("0x01020304", mac_alg_p.prettyPrint()[:10])


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# Use of PBMAC1 in the PKCS #12 Syntax
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9579.txt
#

from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc8018


# Import from RFC 5280

AlgorithmIdentifier = rfc5280.AlgorithmIdentifier


# Import from RFC 8018

rsadsi = rfc8018.rsadsi

pkcs = rfc8018.pkcs

pkcs_5 = rfc8018.pkcs_5

id_hmacWithSHA1 = rfc8018.id_hmacWithSHA1

id_hmacWithSHA224 = rfc8018.id_hmacWithSHA224

id_hmacWithSHA256 = rfc8018.id_hmacWithSHA256

id_hmacWithSHA384 = rfc8018.id_hmacWithSHA384

id_hmacWithSHA512 = rfc8018.id_hmacWithSHA512

id_hmacWithSHA512_224 = rfc8018.id_hmacWithSHA512_224

id_hmacWithSHA512_256 = rfc8018.id_hmacWithSHA512_256

id_PBMAC1 = rfc8018.id_PBMAC1

PBMAC1_params = rfc8018.PBMAC1_params

id_PBKDF2 = rfc8018.id_PBKDF2

PBKDF2_params = rfc8018.PBKDF2_params


# HMAC algorithm identifiers

algid_hmacWithSHA1 = AlgorithmIdentifier()
algid_hmacWithSHA1['algorithm'] = id_hmacWithSHA1
algid_hmacWithSHA1['parameters'] = univ.Null("")

algid_hmacWithSHA224 = AlgorithmIdentifier()
algid_hmacWithSHA224['algorithm'] = id_hmacWithSHA224
algid_hmacWithSHA224['parameters'] = univ.Null("")

algid_hmacWithSHA256 = AlgorithmIdentifier()
algid_hmacWithSHA256['algorithm'] = id_hmacWithSHA256
algid_hmacWithSHA256['parameters'] = univ.Null("")

algid_hmacWithSHA384 = AlgorithmIdentifier()
algid_hmacWithSHA384['algorithm'] = id_hmacWithSHA384
algid_hmacWithSHA384['parameters'] = univ.Null("")

algid_hmacWithSHA512 = AlgorithmIdentifier()
algid_hmacWithSHA512['algorithm'] = id_hmacWithSHA512
algid_hmacWithSHA512['parameters'] = univ.Null("")

algid_hmacWithSHA512_224 = AlgorithmIdentifier()
algid_hmacWithSHA512_224['algorithm'] = id_hmacWithSHA512_224
algid_hmacWithSHA512_224['parameters'] = univ.Null("")

algid_hmacWithSHA512_256 = AlgorithmIdentifier()
algid_hmacWithSHA512_256['algorithm'] = id_hmacWithSHA512_256
algid_hmacWithSHA512_256['parameters'] = univ.Null("")


# No need to Update the Algorithm Identifier map or the S/MIME Capabilities map;
# these updates were handled by importing RFC 8018.



#
# This file is part of pyasn1-alt-modules software.
#
# Updated by Russ Housley to resolve the TODO regarding the Certificate
#   Policies Certificate Extension.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# X.509 message syntax
#
# ASN.1 source from:
# http://www.trl.ibm.com/projects/xml/xss4j/data/asn1/grammars/x509.asn
# http://www.ietf.org/rfc/rfc2459.txt
#
# Sample captures from:
# http://wiki.wireshark.org/SampleCaptures/
#
from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import namedval
from pyasn1.type import opentype
from pyasn1.type import tag
from pyasn1.type import univ
from pyasn1.type import useful

MAX = float('inf')

#
# PKIX1Explicit88
#

# Upper Bounds
ub_name = univ.Integer(32768)
ub_common_name = univ.Integer(64)
ub_locality_name = univ.Integer(128)
ub_state_name = univ.Integer(128)
ub_organization_name = univ.Integer(64)
ub_organizational_unit_name = univ.Integer(64)
ub_title = univ.Integer(64)
ub_match = univ.Integer(128)
ub_emailaddress_length = univ.Integer(128)
ub_common_name_length = univ.Integer(64)
ub_country_name_alpha_length = univ.Integer(2)
ub_country_name_numeric_length = univ.Integer(3)
ub_domain_defined_attributes = univ.Integer(4)
ub_domain_defined_attribute_type_length = univ.Integer(8)
ub_domain_defined_attribute_value_length = univ.Integer(128)
ub_domain_name_length = univ.Integer(16)
ub_extension_attributes = univ.Integer(256)
ub_e163_4_number_length = univ.Integer(15)
ub_e163_4_sub_address_length = univ.Integer(40)
ub_generation_qualifier_length = univ.Integer(3)
ub_given_name_length = univ.Integer(16)
ub_initials_length = univ.Integer(5)
ub_integer_options = univ.Integer(256)
ub_numeric_user_id_length = univ.Integer(32)
ub_organization_name_length = univ.Integer(64)
ub_organizational_unit_name_length = univ.Integer(32)
ub_organizational_units = univ.Integer(4)
ub_pds_name_length = univ.Integer(16)
ub_pds_parameter_length = univ.Integer(30)
ub_pds_physical_address_lines = univ.Integer(6)
ub_postal_code_length = univ.Integer(16)
ub_surname_length = univ.Integer(40)
ub_terminal_id_length = univ.Integer(24)
ub_unformatted_address_length = univ.Integer(180)
ub_x121_address_length = univ.Integer(16)


class UniversalString(char.UniversalString):
    pass


class BMPString(char.BMPString):
    pass


class UTF8String(char.UTF8String):
    pass


id_pkix = univ.ObjectIdentifier('1.3.6.1.5.5.7')
id_pe = univ.ObjectIdentifier('1.3.6.1.5.5.7.1')
id_qt = univ.ObjectIdentifier('1.3.6.1.5.5.7.2')
id_kp = univ.ObjectIdentifier('1.3.6.1.5.5.7.3')
id_ad = univ.ObjectIdentifier('1.3.6.1.5.5.7.48')

id_qt_cps = univ.ObjectIdentifier('1.3.6.1.5.5.7.2.1')
id_qt_unotice = univ.ObjectIdentifier('1.3.6.1.5.5.7.2.2')

id_ad_ocsp = univ.ObjectIdentifier('1.3.6.1.5.5.7.48.1')
id_ad_caIssuers = univ.ObjectIdentifier('1.3.6.1.5.5.7.48.2')




id_at = univ.ObjectIdentifier('2.5.4')
id_at_name = univ.ObjectIdentifier('2.5.4.41')
# preserve misspelled variable for compatibility
id_at_sutname = id_at_surname = univ.ObjectIdentifier('2.5.4.4')
id_at_givenName = univ.ObjectIdentifier('2.5.4.42')
id_at_initials = univ.ObjectIdentifier('2.5.4.43')
id_at_generationQualifier = univ.ObjectIdentifier('2.5.4.44')


class X520name(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString',
                            char.TeletexString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_name))),
        namedtype.NamedType('printableString',
                            char.PrintableString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_name))),
        namedtype.NamedType('universalString',
                            char.UniversalString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_name))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_name))),
        namedtype.NamedType('bmpString',
                            char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_name)))
    )


id_at_commonName = univ.ObjectIdentifier('2.5.4.3')


class X520CommonName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name))),
        namedtype.NamedType('printableString', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name))),
        namedtype.NamedType('universalString', char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name))),
        namedtype.NamedType('bmpString',
                            char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_common_name)))
    )


id_at_localityName = univ.ObjectIdentifier('2.5.4.7')


class X520LocalityName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name))),
        namedtype.NamedType('printableString', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name))),
        namedtype.NamedType('universalString', char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name))),
        namedtype.NamedType('bmpString',
                            char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_locality_name)))
    )


id_at_stateOrProvinceName = univ.ObjectIdentifier('2.5.4.8')


class X520StateOrProvinceName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString',
                            char.TeletexString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name))),
        namedtype.NamedType('printableString', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name))),
        namedtype.NamedType('universalString', char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name))),
        namedtype.NamedType('bmpString',
                            char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_state_name)))
    )


id_at_organizationName = univ.ObjectIdentifier('2.5.4.10')


class X520OrganizationName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name))),
        namedtype.NamedType('printableString', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name))),
        namedtype.NamedType('universalString', char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name))),
        namedtype.NamedType('utf8String', char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name))),
        namedtype.NamedType('bmpString', char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organization_name)))
    )


id_at_organizationalUnitName = univ.ObjectIdentifier('2.5.4.11')


class X520OrganizationalUnitName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name))),
        namedtype.NamedType('printableString', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name))),
        namedtype.NamedType('universalString', char.UniversalString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name))),
        namedtype.NamedType('utf8String', char.UTF8String().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name))),
        namedtype.NamedType('bmpString', char.BMPString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_organizational_unit_name)))
    )


id_at_title = univ.ObjectIdentifier('2.5.4.12')


class X520Title(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString',
                            char.TeletexString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_title))),
        namedtype.NamedType('printableString',
                            char.PrintableString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_title))),
        namedtype.NamedType('universalString',
                            char.UniversalString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_title))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_title))),
        namedtype.NamedType('bmpString',
                            char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, ub_title)))
    )


id_at_dnQualifier = univ.ObjectIdentifier('2.5.4.46')


class X520dnQualifier(char.PrintableString):
    pass


id_at_countryName = univ.ObjectIdentifier('2.5.4.6')


class X520countryName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(2, 2)


pkcs_9 = univ.ObjectIdentifier('1.2.840.113549.1.9')

emailAddress = univ.ObjectIdentifier('1.2.840.113549.1.9.1')


class Pkcs9email(char.IA5String):
    subtypeSpec = char.IA5String.subtypeSpec + constraint.ValueSizeConstraint(1, ub_emailaddress_length)


# ----

class DSAPrivateKey(univ.Sequence):
    """PKIX compliant DSA private key structure"""
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('version', univ.Integer(namedValues=namedval.NamedValues(('v1', 0)))),
        namedtype.NamedType('p', univ.Integer()),
        namedtype.NamedType('q', univ.Integer()),
        namedtype.NamedType('g', univ.Integer()),
        namedtype.NamedType('public', univ.Integer()),
        namedtype.NamedType('private', univ.Integer())
    )


# ----


class DirectoryString(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('teletexString',
                            char.TeletexString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('printableString',
                            char.PrintableString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('universalString',
                            char.UniversalString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('utf8String',
                            char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('bmpString', char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('ia5String', char.IA5String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, MAX)))
        # hm, this should not be here!? XXX
    )


# certificate and CRL specific structures begin here

class AlgorithmIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('algorithm', univ.ObjectIdentifier()),
        namedtype.OptionalNamedType('parameters', univ.Any())
    )



# Algorithm OIDs and parameter structures

pkcs_1 = univ.ObjectIdentifier('1.2.840.113549.1.1')
rsaEncryption = univ.ObjectIdentifier('1.2.840.113549.1.1.1')
md2WithRSAEncryption = univ.ObjectIdentifier('1.2.840.113549.1.1.2')
md5WithRSAEncryption = univ.ObjectIdentifier('1.2.840.113549.1.1.4')
sha1WithRSAEncryption = univ.ObjectIdentifier('1.2.840.113549.1.1.5')
id_dsa_with_sha1 = univ.ObjectIdentifier('1.2.840.10040.4.3')


class Dss_Sig_Value(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('r', univ.Integer()),
        namedtype.NamedType('s', univ.Integer())
    )


dhpublicnumber = univ.ObjectIdentifier('1.2.840.10046.2.1')


class ValidationParms(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('seed', univ.BitString()),
        namedtype.NamedType('pgenCounter', univ.Integer())
    )


class DomainParameters(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('p', univ.Integer()),
        namedtype.NamedType('g', univ.Integer()),
        namedtype.NamedType('q', univ.Integer()),
        namedtype.NamedType('j', univ.Integer()),
        namedtype.OptionalNamedType('validationParms', ValidationParms())
    )


id_dsa = univ.ObjectIdentifier('1.2.840.10040.4.1')


class Dss_Parms(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('p', univ.Integer()),
        namedtype.NamedType('q', univ.Integer()),
        namedtype.NamedType('g', univ.Integer())
    )


# x400 address syntax starts here

teletex_domain_defined_attributes = univ.Integer(6)


class TeletexDomainDefinedAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_defined_attribute_type_length))),
        namedtype.NamedType('value', char.TeletexString())
    )


class TeletexDomainDefinedAttributes(univ.SequenceOf):
    componentType = TeletexDomainDefinedAttribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, ub_domain_defined_attributes)


terminal_type = univ.Integer(23)


class TerminalType(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(0, ub_integer_options)
    namedValues = namedval.NamedValues(
        ('telex', 3),
        ('teletelex', 4),
        ('g3-facsimile', 5),
        ('g4-facsimile', 6),
        ('ia5-terminal', 7),
        ('videotex', 8)
    )


class PresentationAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('pSelector', univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('sSelector', univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('tSelector', univ.OctetString().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('nAddresses', univ.SetOf(componentType=univ.OctetString()).subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3),
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
    )


extended_network_address = univ.Integer(22)


class E163_4_address(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('number', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_e163_4_number_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('sub-address', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_e163_4_sub_address_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)))
    )


class ExtendedNetworkAddress(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('e163-4-address', E163_4_address()),
        namedtype.NamedType('psap-address', PresentationAddress().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)))
    )


class PDSParameter(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('printable-string', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length))),
        namedtype.OptionalNamedType('teletex-string', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)))
    )


local_postal_attributes = univ.Integer(21)


class LocalPostalAttributes(PDSParameter):
    pass


class UniquePostalName(PDSParameter):
    pass


unique_postal_name = univ.Integer(20)

poste_restante_address = univ.Integer(19)


class PosteRestanteAddress(PDSParameter):
    pass


post_office_box_address = univ.Integer(18)


class PostOfficeBoxAddress(PDSParameter):
    pass


street_address = univ.Integer(17)


class StreetAddress(PDSParameter):
    pass


class UnformattedPostalAddress(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('printable-address', univ.SequenceOf(componentType=char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_parameter_length)).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_pds_physical_address_lines)))),
        namedtype.OptionalNamedType('teletex-string', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_unformatted_address_length)))
    )


physical_delivery_office_name = univ.Integer(10)


class PhysicalDeliveryOfficeName(PDSParameter):
    pass


physical_delivery_office_number = univ.Integer(11)


class PhysicalDeliveryOfficeNumber(PDSParameter):
    pass


extension_OR_address_components = univ.Integer(12)


class ExtensionORAddressComponents(PDSParameter):
    pass


physical_delivery_personal_name = univ.Integer(13)


class PhysicalDeliveryPersonalName(PDSParameter):
    pass


physical_delivery_organization_name = univ.Integer(14)


class PhysicalDeliveryOrganizationName(PDSParameter):
    pass


extension_physical_delivery_address_components = univ.Integer(15)


class ExtensionPhysicalDeliveryAddressComponents(PDSParameter):
    pass


unformatted_postal_address = univ.Integer(16)

postal_code = univ.Integer(9)


class PostalCode(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('numeric-code', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length))),
        namedtype.NamedType('printable-code', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_postal_code_length)))
    )


class PhysicalDeliveryCountryName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('x121-dcc-code', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(ub_country_name_numeric_length,
                                                       ub_country_name_numeric_length))),
        namedtype.NamedType('iso-3166-alpha2-code', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(ub_country_name_alpha_length, ub_country_name_alpha_length)))
    )


class PDSName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_pds_name_length)


physical_delivery_country_name = univ.Integer(8)


class TeletexOrganizationalUnitName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_organizational_unit_name_length)


pds_name = univ.Integer(7)

teletex_organizational_unit_names = univ.Integer(5)


class TeletexOrganizationalUnitNames(univ.SequenceOf):
    componentType = TeletexOrganizationalUnitName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, ub_organizational_units)


teletex_personal_name = univ.Integer(4)


class TeletexPersonalName(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('surname', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('given-name', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('initials', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('generation-qualifier', char.TeletexString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_generation_qualifier_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)))
    )


teletex_organization_name = univ.Integer(3)


class TeletexOrganizationName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_organization_name_length)


teletex_common_name = univ.Integer(2)


class TeletexCommonName(char.TeletexString):
    subtypeSpec = char.TeletexString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_common_name_length)


class CommonName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_common_name_length)


common_name = univ.Integer(1)


class ExtensionAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('extension-attribute-type', univ.Integer().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(0, ub_extension_attributes),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('extension-attribute-value',
                            univ.Any().subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)))
    )


class ExtensionAttributes(univ.SetOf):
    componentType = ExtensionAttribute()
    sizeSpec = univ.SetOf.sizeSpec + constraint.ValueSizeConstraint(1, ub_extension_attributes)


class BuiltInDomainDefinedAttribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_defined_attribute_type_length))),
        namedtype.NamedType('value', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_defined_attribute_value_length)))
    )


class BuiltInDomainDefinedAttributes(univ.SequenceOf):
    componentType = BuiltInDomainDefinedAttribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, ub_domain_defined_attributes)


class OrganizationalUnitName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_organizational_unit_name_length)


class OrganizationalUnitNames(univ.SequenceOf):
    componentType = OrganizationalUnitName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, ub_organizational_units)


class PersonalName(univ.Set):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('surname', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_surname_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('given-name', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_given_name_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('initials', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_initials_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('generation-qualifier', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_generation_qualifier_length),
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)))
    )


class NumericUserIdentifier(char.NumericString):
    subtypeSpec = char.NumericString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_numeric_user_id_length)


class OrganizationName(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_organization_name_length)


class PrivateDomainName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('numeric', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length))),
        namedtype.NamedType('printable', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, ub_domain_name_length)))
    )


class TerminalIdentifier(char.PrintableString):
    subtypeSpec = char.PrintableString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_terminal_id_length)


class X121Address(char.NumericString):
    subtypeSpec = char.NumericString.subtypeSpec + constraint.ValueSizeConstraint(1, ub_x121_address_length)


class NetworkAddress(X121Address):
    pass


class AdministrationDomainName(univ.Choice):
    tagSet = univ.Choice.tagSet.tagExplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 2)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('numeric', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length))),
        namedtype.NamedType('printable', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(0, ub_domain_name_length)))
    )


class CountryName(univ.Choice):
    tagSet = univ.Choice.tagSet.tagExplicitly(
        tag.Tag(tag.tagClassApplication, tag.tagFormatConstructed, 1)
    )
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('x121-dcc-code', char.NumericString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(ub_country_name_numeric_length,
                                                       ub_country_name_numeric_length))),
        namedtype.NamedType('iso-3166-alpha2-code', char.PrintableString().subtype(
            subtypeSpec=constraint.ValueSizeConstraint(ub_country_name_alpha_length, ub_country_name_alpha_length)))
    )


class BuiltInStandardAttributes(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('country-name', CountryName()),
        namedtype.OptionalNamedType('administration-domain-name', AdministrationDomainName()),
        namedtype.OptionalNamedType('network-address', NetworkAddress().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('terminal-identifier', TerminalIdentifier().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('private-domain-name', PrivateDomainName().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('organization-name', OrganizationName().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.OptionalNamedType('numeric-user-identifier', NumericUserIdentifier().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4))),
        namedtype.OptionalNamedType('personal-name', PersonalName().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5))),
        namedtype.OptionalNamedType('organizational-unit-names', OrganizationalUnitNames().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6)))
    )


class ORAddress(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('built-in-standard-attributes', BuiltInStandardAttributes()),
        namedtype.OptionalNamedType('built-in-domain-defined-attributes', BuiltInDomainDefinedAttributes()),
        namedtype.OptionalNamedType('extension-attributes', ExtensionAttributes())
    )


#
# PKIX1Implicit88
#

id_ce_invalidityDate = univ.ObjectIdentifier('2.5.29.24')


class InvalidityDate(useful.GeneralizedTime):
    pass


id_holdinstruction_none = univ.ObjectIdentifier('2.2.840.10040.2.1')
id_holdinstruction_callissuer = univ.ObjectIdentifier('2.2.840.10040.2.2')
id_holdinstruction_reject = univ.ObjectIdentifier('2.2.840.10040.2.3')

holdInstruction = univ.ObjectIdentifier('2.2.840.10040.2')

id_ce_holdInstructionCode = univ.ObjectIdentifier('2.5.29.23')


class HoldInstructionCode(univ.ObjectIdentifier):
    pass


id_ce_cRLReasons = univ.ObjectIdentifier('2.5.29.21')


class CRLReason(univ.Enumerated):
    namedValues = namedval.NamedValues(
        ('unspecified', 0),
        ('keyCompromise', 1),
        ('cACompromise', 2),
        ('affiliationChanged', 3),
        ('superseded', 4),
        ('cessationOfOperation', 5),
        ('certificateHold', 6),
        ('removeFromCRL', 8)
    )


id_ce_cRLNumber = univ.ObjectIdentifier('2.5.29.20')


class CRLNumber(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(0, MAX)


class BaseCRLNumber(CRLNumber):
    pass


id_kp_serverAuth = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.1')
id_kp_clientAuth = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.2')
id_kp_codeSigning = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.3')
id_kp_emailProtection = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.4')
id_kp_ipsecEndSystem = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.5')
id_kp_ipsecTunnel = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.6')
id_kp_ipsecUser = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.7')
id_kp_timeStamping = univ.ObjectIdentifier('1.3.6.1.5.5.7.3.8')
id_pe_authorityInfoAccess = univ.ObjectIdentifier('1.3.6.1.5.5.7.1.1')
id_ce_extKeyUsage = univ.ObjectIdentifier('2.5.29.37')


class KeyPurposeId(univ.ObjectIdentifier):
    pass


class ExtKeyUsageSyntax(univ.SequenceOf):
    componentType = KeyPurposeId()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class ReasonFlags(univ.BitString):
    namedValues = namedval.NamedValues(
        ('unused', 0),
        ('keyCompromise', 1),
        ('cACompromise', 2),
        ('affiliationChanged', 3),
        ('superseded', 4),
        ('cessationOfOperation', 5),
        ('certificateHold', 6)
    )


class SkipCerts(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueSizeConstraint(0, MAX)


id_ce_policyConstraints = univ.ObjectIdentifier('2.5.29.36')


class PolicyConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('requireExplicitPolicy', SkipCerts().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('inhibitPolicyMapping', SkipCerts().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)))
    )


id_ce_basicConstraints = univ.ObjectIdentifier('2.5.29.19')


class BasicConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('cA', univ.Boolean(False)),
        namedtype.OptionalNamedType('pathLenConstraint',
                                    univ.Integer().subtype(subtypeSpec=constraint.ValueRangeConstraint(0, MAX)))
    )


id_ce_subjectDirectoryAttributes = univ.ObjectIdentifier('2.5.29.9')


class EDIPartyName(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('nameAssigner', DirectoryString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('partyName',
                            DirectoryString().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)))
    )



id_ce_deltaCRLIndicator = univ.ObjectIdentifier('2.5.29.27')



class BaseDistance(univ.Integer):
    subtypeSpec = univ.Integer.subtypeSpec + constraint.ValueRangeConstraint(0, MAX)


id_ce_cRLDistributionPoints = univ.ObjectIdentifier('2.5.29.31')


id_ce_issuingDistributionPoint = univ.ObjectIdentifier('2.5.29.28')




id_ce_nameConstraints = univ.ObjectIdentifier('2.5.29.30')


class DisplayText(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('visibleString',
                            char.VisibleString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200))),
        namedtype.NamedType('bmpString', char.BMPString().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200))),
        namedtype.NamedType('utf8String', char.UTF8String().subtype(subtypeSpec=constraint.ValueSizeConstraint(1, 200)))
    )


class NoticeReference(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('organization', DisplayText()),
        namedtype.NamedType('noticeNumbers', univ.SequenceOf(componentType=univ.Integer()))
    )


class UserNotice(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('noticeRef', NoticeReference()),
        namedtype.OptionalNamedType('explicitText', DisplayText())
    )


class CPSuri(char.IA5String):
    pass


class PolicyQualifierId(univ.ObjectIdentifier):
    subtypeSpec = univ.ObjectIdentifier.subtypeSpec + constraint.SingleValueConstraint(id_qt_cps, id_qt_unotice)


class CertPolicyId(univ.ObjectIdentifier):
    pass


class PolicyQualifierInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('policyQualifierId', PolicyQualifierId()),
        namedtype.NamedType('qualifier', univ.Any())
    )


id_ce_certificatePolicies = univ.ObjectIdentifier('2.5.29.32')


class PolicyInformation(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('policyIdentifier', CertPolicyId()),
        namedtype.OptionalNamedType('policyQualifiers', univ.SequenceOf(componentType=PolicyQualifierInfo()).subtype(
            subtypeSpec=constraint.ValueSizeConstraint(1, MAX)))
    )


class CertificatePolicies(univ.SequenceOf):
    componentType = PolicyInformation()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


id_ce_policyMappings = univ.ObjectIdentifier('2.5.29.33')


class PolicyMapping(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('issuerDomainPolicy', CertPolicyId()),
        namedtype.NamedType('subjectDomainPolicy', CertPolicyId())
    )


class PolicyMappings(univ.SequenceOf):
    componentType = PolicyMapping()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


id_ce_privateKeyUsagePeriod = univ.ObjectIdentifier('2.5.29.16')


class PrivateKeyUsagePeriod(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('notBefore', useful.GeneralizedTime().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('notAfter', useful.GeneralizedTime().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1)))
    )


id_ce_keyUsage = univ.ObjectIdentifier('2.5.29.15')


class KeyUsage(univ.BitString):
    namedValues = namedval.NamedValues(
        ('digitalSignature', 0),
        ('nonRepudiation', 1),
        ('keyEncipherment', 2),
        ('dataEncipherment', 3),
        ('keyAgreement', 4),
        ('keyCertSign', 5),
        ('cRLSign', 6),
        ('encipherOnly', 7),
        ('decipherOnly', 8)
    )


id_ce = univ.ObjectIdentifier('2.5.29')

id_ce_authorityKeyIdentifier = univ.ObjectIdentifier('2.5.29.35')


class KeyIdentifier(univ.OctetString):
    pass


id_ce_subjectKeyIdentifier = univ.ObjectIdentifier('2.5.29.14')


class SubjectKeyIdentifier(KeyIdentifier):
    pass


id_ce_certificateIssuer = univ.ObjectIdentifier('2.5.29.29')


id_ce_subjectAltName = univ.ObjectIdentifier('2.5.29.17')


id_ce_issuerAltName = univ.ObjectIdentifier('2.5.29.18')


class AttributeValue(univ.Any):
    pass


class AttributeType(univ.ObjectIdentifier):
    pass

certificateAttributesMap = {}


class AttributeTypeAndValue(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', AttributeType()),
        namedtype.NamedType('value', AttributeValue(),
                            openType=opentype.OpenType('type', certificateAttributesMap))
    )


class Attribute(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type', AttributeType()),
        namedtype.NamedType('vals', univ.SetOf(componentType=AttributeValue()))
    )


class SubjectDirectoryAttributes(univ.SequenceOf):
    componentType = Attribute()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class RelativeDistinguishedName(univ.SetOf):
    componentType = AttributeTypeAndValue()


class RDNSequence(univ.SequenceOf):
    componentType = RelativeDistinguishedName()


class Name(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('', RDNSequence())
    )

class CertificateSerialNumber(univ.Integer):
    pass


class AnotherName(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('type-id', univ.ObjectIdentifier()),
        namedtype.NamedType('value',
                            univ.Any().subtype(explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0)))
    )


class GeneralName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('otherName',
                            AnotherName().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('rfc822Name',
                            char.IA5String().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.NamedType('dNSName',
                            char.IA5String().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.NamedType('x400Address',
                            ORAddress().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.NamedType('directoryName',
                            Name().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4))),
        namedtype.NamedType('ediPartyName',
                            EDIPartyName().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 5))),
        namedtype.NamedType('uniformResourceIdentifier',
                            char.IA5String().subtype(implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 6))),
        namedtype.NamedType('iPAddress', univ.OctetString().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 7))),
        namedtype.NamedType('registeredID', univ.ObjectIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 8)))
    )


class GeneralNames(univ.SequenceOf):
    componentType = GeneralName()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class AccessDescription(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('accessMethod', univ.ObjectIdentifier()),
        namedtype.NamedType('accessLocation', GeneralName())
    )


class AuthorityInfoAccessSyntax(univ.SequenceOf):
    componentType = AccessDescription()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class AuthorityKeyIdentifier(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('keyIdentifier', KeyIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.OptionalNamedType('authorityCertIssuer', GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('authorityCertSerialNumber', CertificateSerialNumber().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2)))
    )


class DistributionPointName(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('fullName', GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.NamedType('nameRelativeToCRLIssuer', RelativeDistinguishedName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)))
    )


class DistributionPoint(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('distributionPoint', DistributionPointName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('reasons', ReasonFlags().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('cRLIssuer', GeneralNames().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 2)))
    )


class CRLDistPointsSyntax(univ.SequenceOf):
    componentType = DistributionPoint()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class IssuingDistributionPoint(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('distributionPoint', DistributionPointName().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.NamedType('onlyContainsUserCerts', univ.Boolean(False).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.NamedType('onlyContainsCACerts', univ.Boolean(False).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('onlySomeReasons', ReasonFlags().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3))),
        namedtype.NamedType('indirectCRL', univ.Boolean(False).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)))
    )


class GeneralSubtree(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('base', GeneralName()),
        namedtype.DefaultedNamedType('minimum', BaseDistance(0).subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('maximum', BaseDistance().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)))
    )


class GeneralSubtrees(univ.SequenceOf):
    componentType = GeneralSubtree()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class NameConstraints(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('permittedSubtrees', GeneralSubtrees().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('excludedSubtrees', GeneralSubtrees().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 1)))
    )


class CertificateIssuer(GeneralNames):
    pass


class SubjectAltName(GeneralNames):
    pass


class IssuerAltName(GeneralNames):
    pass


certificateExtensionsMap = {}


class Extension(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('extnID', univ.ObjectIdentifier()),
        namedtype.DefaultedNamedType('critical', univ.Boolean('False')),
        namedtype.NamedType('extnValue', univ.OctetString(),
                            openType=opentype.OpenType('extnID', certificateExtensionsMap))
    )


class Extensions(univ.SequenceOf):
    componentType = Extension()
    sizeSpec = univ.SequenceOf.sizeSpec + constraint.ValueSizeConstraint(1, MAX)


class SubjectPublicKeyInfo(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('algorithm', AlgorithmIdentifier()),
        namedtype.NamedType('subjectPublicKey', univ.BitString())
    )


class UniqueIdentifier(univ.BitString):
    pass


class Time(univ.Choice):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('utcTime', useful.UTCTime()),
        namedtype.NamedType('generalTime', useful.GeneralizedTime())
    )


class Validity(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('notBefore', Time()),
        namedtype.NamedType('notAfter', Time())
    )


class Version(univ.Integer):
    namedValues = namedval.NamedValues(
        ('v1', 0), ('v2', 1), ('v3', 2)
    )


class TBSCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('version', Version('v1').subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 0))),
        namedtype.NamedType('serialNumber', CertificateSerialNumber()),
        namedtype.NamedType('signature', AlgorithmIdentifier()),
        namedtype.NamedType('issuer', Name()),
        namedtype.NamedType('validity', Validity()),
        namedtype.NamedType('subject', Name()),
        namedtype.NamedType('subjectPublicKeyInfo', SubjectPublicKeyInfo()),
        namedtype.OptionalNamedType('issuerUniqueID', UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 1))),
        namedtype.OptionalNamedType('subjectUniqueID', UniqueIdentifier().subtype(
            implicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 2))),
        namedtype.OptionalNamedType('extensions', Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 3)))
    )


class Certificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('tbsCertificate', TBSCertificate()),
        namedtype.NamedType('signatureAlgorithm', AlgorithmIdentifier()),
        namedtype.NamedType('signatureValue', univ.BitString())
    )

# CRL structures

class RevokedCertificate(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('userCertificate', CertificateSerialNumber()),
        namedtype.NamedType('revocationDate', Time()),
        namedtype.OptionalNamedType('crlEntryExtensions', Extensions())
    )


class TBSCertList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.OptionalNamedType('version', Version()),
        namedtype.NamedType('signature', AlgorithmIdentifier()),
        namedtype.NamedType('issuer', Name()),
        namedtype.NamedType('thisUpdate', Time()),
        namedtype.OptionalNamedType('nextUpdate', Time()),
        namedtype.OptionalNamedType('revokedCertificates', univ.SequenceOf(componentType=RevokedCertificate())),
        namedtype.OptionalNamedType('crlExtensions', Extensions().subtype(
            explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatConstructed, 0)))
    )


class CertificateList(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('tbsCertList', TBSCertList()),
        namedtype.NamedType('signatureAlgorithm', AlgorithmIdentifier()),
        namedtype.NamedType('signature', univ.BitString())
    )

# map of AttributeType -> AttributeValue

_certificateAttributesMapUpdate = {
    id_at_name: X520name(),
    id_at_surname: X520name(),
    id_at_givenName: X520name(),
    id_at_initials: X520name(),
    id_at_generationQualifier: X520name(),
    id_at_commonName: X520CommonName(),
    id_at_localityName: X520LocalityName(),
    id_at_stateOrProvinceName: X520StateOrProvinceName(),
    id_at_organizationName: X520OrganizationName(),
    id_at_organizationalUnitName: X520OrganizationalUnitName(),
    id_at_title: X520Title(),
    id_at_dnQualifier: X520dnQualifier(),
    id_at_countryName: X520countryName(),
    emailAddress: Pkcs9email(),
}

certificateAttributesMap.update(_certificateAttributesMapUpdate)


# map of Certificate Extension OIDs to Extensions

_certificateExtensionsMapUpdate = {
    id_ce_authorityKeyIdentifier: AuthorityKeyIdentifier(),
    id_ce_subjectKeyIdentifier: SubjectKeyIdentifier(),
    id_ce_keyUsage: KeyUsage(),
    id_ce_privateKeyUsagePeriod: PrivateKeyUsagePeriod(),
    id_ce_certificatePolicies: CertificatePolicies(),
    id_ce_policyMappings: PolicyMappings(),
    id_ce_subjectAltName: SubjectAltName(),
    id_ce_issuerAltName: IssuerAltName(),
    id_ce_subjectDirectoryAttributes: SubjectDirectoryAttributes(),
    id_ce_basicConstraints: BasicConstraints(),
    id_ce_nameConstraints: NameConstraints(),
    id_ce_policyConstraints: PolicyConstraints(),
    id_ce_extKeyUsage: ExtKeyUsageSyntax(),
    id_ce_cRLDistributionPoints: CRLDistPointsSyntax(),
    id_pe_authorityInfoAccess: AuthorityInfoAccessSyntax(),
    id_ce_cRLNumber: univ.Integer(),
    id_ce_deltaCRLIndicator: BaseCRLNumber(),
    id_ce_issuingDistributionPoint: IssuingDistributionPoint(),
    id_ce_cRLReasons: CRLReason(),
    id_ce_holdInstructionCode: univ.ObjectIdentifier(),
    id_ce_invalidityDate: useful.GeneralizedTime(),
    id_ce_certificateIssuer: GeneralNames(),
}

certificateExtensionsMap.update(_certificateExtensionsMapUpdate)




#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import rfc9608


class NoRevAvailCertificateTestCase(unittest.TestCase):
    cert_pem_text = """\
MIICizCCAhGgAwIBAgIJAKWzVCgbsG5RMAoGCCqGSM49BAMDMD8xCzAJBgNVBAYT
AlVTMQswCQYDVQQIDAJWQTEQMA4GA1UEBwwHSGVybmRvbjERMA8GA1UECgwIQm9n
dXMgQ0EwHhcNMjMwNTE5MTc1NzMwWhcNMjMwNTI2MTc1NzMwWjA/MRgwFgYDVQQD
Ew93d3cuZXhhbXBsZS5jb20xFjAUBgNVBAoTDUV4YW1wbGUgQ29ycC4xCzAJBgNV
BAYTAlVTMHYwEAYHKoZIzj0CAQYFK4EEACIDYgAEVyVaFAmNjlVGE8vcwnSqa6t6
4CLfG4iaYazd9dZrUPyD1/7P9CcR+881jkrRMMrgKiZh0v/xkdwdt4NhI4L6I4vE
WqDmO0IAc+FT5yyurBWMkIUildRvDrPoCogafOJso4HYMIHVMAkGA1UdOAQCBQAw
IgYDVR0RBBswGYYXaHR0cDovL3d3dy5leGFtcGxlLmNvbS8wCwYDVR0PBAQDAgeA
MBMGA1UdJQQMMAoGCCsGAQUFBwMBMEIGCWCGSAGG+EIBDQQ1FjNUaGlzIGNlcnRp
ZmljYXRlIGNhbm5vdCBiZSB0cnVzdGVkIGZvciBhbnkgcHVycG9zZS4wHQYDVR0O
BBYEFIQ6BKw0c2EZOZq2RWrTfGjTXz3cMB8GA1UdIwQYMBaAFPI12zQE2qVV8r1p
A5mwYuziFQjBMAoGCCqGSM49BAMDA2gAMGUCMQDSz7l1NptMer/vrjpvhpGXNBNH
5C99shVOvKYlkfn7BCvYmvGG6VaR7xiT4uhbd1ECMERXfNTrV0TDuB4RHYfsJ2sL
hsc0TPUNfghSWecand/COC4SDVid/Xyg7A1DQrzTvg==
"""

    def setUp(self):
        self.asn1Spec = rfc5280.Certificate()

    def testDerCodec(self):
        substrate = pem.readBase64fromText(self.cert_pem_text)
        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))

        found = False
        for extn in asn1Object['tbsCertificate']['extensions']:
            if extn['extnID'] in rfc5280.certificateExtensionsMap:
                extnValue, rest = der_decoder(
                    extn['extnValue'],
                    asn1Spec=rfc5280.certificateExtensionsMap[extn['extnID']])
                self.assertFalse(rest)
                self.assertEqual(extn['extnValue'], der_encoder(extnValue))

                if extn['extnID'] == rfc9608.id_ce_noRevAvail:
                    found = True

        self.assertTrue(found)

suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    unittest.TextTestRunner(verbosity=2).run(suite)



#
# This file is part of pyasn1-alt-modules software.
#
# Copyright (c) 2005-2020, Ilya Etingof <etingof@gmail.com>
# Copyright (c) 2021-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
import sys
import unittest

from pyasn1.codec.der.decoder import decode as der_decoder
from pyasn1.codec.der.encoder import encode as der_encoder

from pyasn1_alt_modules import pem
from pyasn1_alt_modules import rfc2314


class CertificationRequestTestCase(unittest.TestCase):
    pem_text = """\
MIIDATCCAekCAQAwgZkxCzAJBgNVBAYTAlJVMRYwFAYDVQQIEw1Nb3Njb3cgUmVn
aW9uMQ8wDQYDVQQHEwZNb3Njb3cxGjAYBgNVBAoTEVNOTVAgTGFib3JhdG9yaWVz
MQwwCgYDVQQLFANSJkQxFTATBgNVBAMTDHNubXBsYWJzLmNvbTEgMB4GCSqGSIb3
DQEJARYRaW5mb0Bzbm1wbGFicy5jb20wggEiMA0GCSqGSIb3DQEBAQUAA4IBDwAw
ggEKAoIBAQC9n2NfGS98JDBmAXQn+vNUyPB3QPYC1cwpX8UMYh9MdAmBZJCnvXrQ
Pp14gNAv6AQKxefmGES1b+Yd+1we9HB8AKm1/8xvRDUjAvy4iO0sqFCPvIfSujUy
pBcfnR7QE2itvyrMxCDSEVnMhKdCNb23L2TptUmpvLcb8wfAMLFsSu2yaOtJysep
oH/mvGqlRv2ti2+E2YA0M7Pf83wyV1XmuEsc9tQ225rprDk2uyshUglkDD2235rf
0QyONq3Aw3BMrO9ss1qj7vdDhVHVsxHnTVbEgrxEWkq2GkVKh9QReMZ2AKxe40j4
og+OjKXguOCggCZHJyXKxccwqCaeCztbAgMBAAGgIjAgBgkqhkiG9w0BCQIxExMR
U05NUCBMYWJvcmF0b3JpZXMwDQYJKoZIhvcNAQEFBQADggEBAAihbwmN9M2bsNNm
9KfxqiGMqqcGCtzIlpDz/2NVwY93cEZsbz3Qscc0QpknRmyTSoDwIG+1nUH0vzkT
Nv8sBmp9I1GdhGg52DIaWwL4t9O5WUHgfHSJpPxZ/zMP2qIsdPJ+8o19BbXRlufc
73c03H1piGeb9VcePIaulSHI622xukI6f4Sis49vkDaoi+jadbEEb6TYkJQ3AMRD
WdApGGm0BePdLqboW1Yv70WRRFFD8sxeT7Yw4qrJojdnq0xMHPGfKpf6dJsqWkHk
b5DRbjil1Zt9pJuF680S9wtBzSi0hsMHXR9TzS7HpMjykL2nmCVY6A78MZapsCzn
GGbx7DI=
"""

    def setUp(self):
        self.asn1Spec = rfc2314.CertificationRequest()

    def testDerCodec(self):

        substrate = pem.readBase64fromText(self.pem_text)

        asn1Object, rest = der_decoder(substrate, asn1Spec=self.asn1Spec)

        self.assertFalse(rest)
        self.assertTrue(asn1Object.prettyPrint())
        self.assertEqual(substrate, der_encoder(asn1Object))


suite = unittest.TestLoader().loadTestsFromModule(sys.modules[__name__])

if __name__ == '__main__':
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    sys.exit(not result.wasSuccessful())



#
# This file is part of pyasn1-alt-modules software.
#
# Created by Russ Housley with some assistance from asn1ate v.0.6.0.
#
# Copyright (c) 2024-2025, Vigil Security, LLC
# License: http://vigilsec.com/pyasn1-alt-modules-license.txt
#
# RPKI Signed Trust Anchor List
#
# ASN.1 source from:
# https://www.rfc-editor.org/rfc/rfc9691.txt
#

from pyasn1.type import char
from pyasn1.type import constraint
from pyasn1.type import namedtype
from pyasn1.type import tag
from pyasn1.type import univ

from pyasn1_alt_modules import rfc5280
from pyasn1_alt_modules import opentypemap

cmsContentTypesMap = opentypemap.get('cmsContentTypesMap')

MAX = float('inf')


# Import from RFC 5280

SubjectPublicKeyInfo = rfc5280.SubjectPublicKeyInfo


# Signed Trust Anchor List
 
class CertificateURI(char.IA5String):
    pass


class TAKey(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.NamedType('comments', univ.SequenceOf(
            componentType=char.UTF8String()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(0, MAX))),
        namedtype.NamedType('certificateURIs', univ.SequenceOf(
            componentType=CertificateURI()).subtype(
                subtypeSpec=constraint.ValueSizeConstraint(1, MAX))),
        namedtype.NamedType('subjectPublicKeyInfo', SubjectPublicKeyInfo())
    )


class TAK(univ.Sequence):
    componentType = namedtype.NamedTypes(
        namedtype.DefaultedNamedType('version',
            univ.Integer().subtype(value=0)),
        namedtype.NamedType('current', TAKey()),
        namedtype.OptionalNamedType('predecessor',
            TAKey().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 0))),
        namedtype.OptionalNamedType('successor',
            TAKey().subtype(explicitTag=tag.Tag(
                tag.tagClassContext, tag.tagFormatConstructed, 1)))
)


id_ct_signedTAL = univ.ObjectIdentifier((1, 2, 840, 113549, 1, 9, 16, 1, 50))


# Update the CMS Content Type Map

_cmsContentTypesMapUpdate = {
    id_ct_signedTAL: TAK(),
}

cmsContentTypesMap.update(_cmsContentTypesMapUpdate)
